@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -9,7 +9,7 @@
 #include "MFace.h"
 #include "Hash.h"
 
-struct Hash_Face : public std::unary_function<MFace, size_t> {
+struct MFaceHash : public std::unary_function<MFace, size_t> {
   size_t operator()(const MFace &f) const
   {
     const MVertex *v[4] = {0, 0, 0, 0};
