@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -319,6 +319,12 @@ public:
                                 12, 14, 13, 8, 10, 11};
     return getVertex(map[num]);
   }
+  virtual MVertex *getVertexVTK(int num)
+  {
+    static const int map[15] = {0, 1, 2, 3, 4, 5, 6, 9,
+                                7, 12, 14, 13, 8, 10, 11};
+    return getVertex(map[num]);
+  }
   virtual MVertex *getVertexKEY(int num) { return getVertexBDF(num); }
   virtual int getNumEdgeVertices() const { return 9; }
   virtual int getNumEdgesRep(bool curved);
@@ -347,6 +353,7 @@ public:
   }
   virtual int getTypeForMSH() const { return MSH_PRI_15; }
   virtual int getTypeForUNV() const { return 113; } // solid parabolic wedge
+  virtual int getTypeForVTK() const { return 26; }
   virtual const char *getStringForBDF() const { return "CPENTA"; }
   virtual const char *getStringForINP() const { return "C3D15"; }
   virtual const char *getStringForKEY() const
@@ -506,6 +513,13 @@ public:
   {
     num < 6 ? MPrism::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
   }
+  virtual int getTypeForVTK() const { return 32; }
+  virtual MVertex *getVertexVTK(int num)
+  {
+    static const int map[18] = {0, 1, 2, 3, 4, 5, 6, 9, 7, 
+                                12, 14, 13, 8, 10, 11, 15,17,16};
+    return getVertex(map[num]);
+  }
 };
 
 /*
