@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -21,11 +21,10 @@ protected:
 public:
   OCCRegion(GModel *m, TopoDS_Solid s, int num);
   virtual ~OCCRegion();
-  virtual SBoundingBox3d bounds(bool fast = false) const;
+  virtual SBoundingBox3d bounds(bool fast = false);
   virtual GeomType geomType() const;
   ModelType getNativeType() const { return OpenCascadeModel; }
   void *getNativePtr() const { return (void *)&s; }
-  TopoDS_Solid getTopoDS_Shape() { return s; }
   void writeBREP(const char *filename);
 };
 
