@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -37,7 +37,7 @@ static double smoothPrimitive(GEdge *ge, double alpha,
 
     // use a gauss-seidel iteration; iterate forward and then backward;
     // convergence is usually very fast
-    for(unsigned int i = 1; i < Points.size(); i++) {
+    for(std::size_t i = 1; i < Points.size(); i++) {
       double dh =
         (Points[i].xp / Points[i].lc - Points[i - 1].xp / Points[i - 1].lc);
       double dt = Points[i].t - Points[i - 1].t;
@@ -93,9 +93,9 @@ struct F_Lc {
     double t_begin = bounds.low();
     double t_end = bounds.high();
     double lc_here;
-    if(t == t_begin)
+    if(t == t_begin && ge->getBeginVertex())
       lc_here = BGM_MeshSize(ge->getBeginVertex(), t, 0, p.x(), p.y(), p.z());
-    else if(t == t_end)
+    else if(t == t_end && ge->getEndVertex())
       lc_here = BGM_MeshSize(ge->getEndVertex(), t, 0, p.x(), p.y(), p.z());
     else
       lc_here = BGM_MeshSize(ge, t, 0, p.x(), p.y(), p.z());
@@ -114,9 +114,9 @@ struct F_Lc_aniso {
     double t_begin = bounds.low();
     double t_end = bounds.high();
 
-    if(t == t_begin)
+    if(t == t_begin && ge->getBeginVertex())
       lc_here = BGM_MeshMetric(ge->getBeginVertex(), t, 0, p.x(), p.y(), p.z());
-    else if(t == t_end)
+    else if(t == t_end && ge->getEndVertex())
       lc_here = BGM_MeshMetric(ge->getEndVertex(), t, 0, p.x(), p.y(), p.z());
     else
       lc_here = BGM_MeshMetric(ge, t, 0, p.x(), p.y(), p.z());
@@ -273,8 +273,25 @@ static double Integration(GEdge *ge, double t1, double t2, function f,
   return Points.back().p;
 }
 
-void copyMesh(GEdge *from, GEdge *to, int direction)
+static SPoint3 transform(MVertex *vsource, const std::vector<double> &tfo)
 {
+  double ps[4] = {vsource->x(), vsource->y(), vsource->z(), 1.};
+  double res[4] = {0., 0., 0., 0.};
+  int idx = 0;
+  for(int i = 0; i < 4; i++)
+    for(int j = 0; j < 4; j++) res[i] += tfo[idx++] * ps[j];
+
+  return SPoint3(res[0], res[1], res[2]);
+}
+
+static void copyMesh(GEdge *from, GEdge *to, int direction)
+{
+  if(!from->getBeginVertex() || !from->getEndVertex() ||
+     !to->getBeginVertex() || !to->getEndVertex()){
+    Msg::Error("Cannot copy mesh on curves without begin/end points");
+    return;
+  }
+
   Range<double> u_bounds = from->parBounds(0);
   double u_min = u_bounds.low();
   double u_max = u_bounds.high();
@@ -283,29 +300,39 @@ void copyMesh(GEdge *from, GEdge *to, int direction)
   double to_u_min = to_u_bounds.low();
 
   // include begin and end point to avoid conflicts when realigning
-
   MVertex *vt0 = to->getBeginVertex()->mesh_vertices[0];
   MVertex *vt1 = to->getEndVertex()->mesh_vertices[0];
-
   MVertex *vs0 = from->getBeginVertex()->mesh_vertices[0];
   MVertex *vs1 = from->getEndVertex()->mesh_vertices[0];
 
   to->correspondingVertices[vt0] = direction > 0 ? vs0 : vs1;
   to->correspondingVertices[vt1] = direction > 0 ? vs1 : vs0;
 
-  for(unsigned int i = 0; i < from->mesh_vertices.size(); i++) {
+  for(std::size_t i = 0; i < from->mesh_vertices.size(); i++) {
     int index = (direction < 0) ? (from->mesh_vertices.size() - 1 - i) : i;
     MVertex *v = from->mesh_vertices[index];
-    double u;
-    v->getParameter(0, u);
-    double newu =
-      (direction > 0) ? (u - u_min + to_u_min) : (u_max - u + to_u_min);
-    GPoint gp = to->point(newu);
+    GPoint gp;
+    double newu;
+    if(to->affineTransform.size() < 16) {
+      // assume identical parametrisations (dangerous...)
+      double u;
+      v->getParameter(0, u);
+      newu =
+        (direction > 0) ? (u - u_min + to_u_min) : (u_max - u + to_u_min);
+      gp = to->point(newu);
+    }
+    else {
+      SPoint3 p = transform(v, to->affineTransform);
+      // for non-exact periodic curves we could use closestPoint; just use
+      // Mesh.HighOrderPeriodic=2, which will do that
+      newu = to->parFromPoint(p);
+      gp = to->point(newu);
+    }
     MEdgeVertex *vv = new MEdgeVertex(gp.x(), gp.y(), gp.z(), to, newu);
     to->mesh_vertices.push_back(vv);
     to->correspondingVertices[vv] = v;
   }
-  for(unsigned int i = 0; i < to->mesh_vertices.size() + 1; i++) {
+  for(std::size_t i = 0; i < to->mesh_vertices.size() + 1; i++) {
     MVertex *v0 = (i == 0) ? to->getBeginVertex()->mesh_vertices[0] :
                              to->mesh_vertices[i - 1];
     MVertex *v1 = (i == to->mesh_vertices.size()) ?
@@ -318,8 +345,9 @@ void copyMesh(GEdge *from, GEdge *to, int direction)
 void deMeshGEdge::operator()(GEdge *ge)
 {
   if(ge->geomType() == GEntity::DiscreteCurve){
-    if(!static_cast<discreteEdge *>(ge)->haveParametrization())
+    if(!static_cast<discreteEdge *>(ge)->haveParametrization()){
       return;
+    }
   }
   ge->deleteMesh();
   ge->meshStatistics.status = GEdge::PENDING;
@@ -335,7 +363,7 @@ static void printFandPrimitive(int tag, std::vector<IntPoint> &Points)
   FILE *f = Fopen(name, "w");
   if(!f) return;
   double l = 0;
-  for (unsigned int i = 0; i < Points.size(); i++){
+  for (std::size_t i = 0; i < Points.size(); i++){
     const IntPoint &P = Points[i];
     if(i) l += (P.t - Points[i-1].t) * P.xp;
     fprintf(f, "%g %g %g %g %g\n", P.t, P.xp/P.lc, P.p, P.lc, l);
@@ -357,7 +385,7 @@ static void filterPoints(GEdge *ge, int nMinimumPoints)
 {
   if(ge->mesh_vertices.empty()) return;
   if(ge->meshAttributes.method == MESH_TRANSFINITE) return;
-  // if(ge->mesh_vertices.size() <= 3) return;
+
   bool forceOdd = false;
   if((ge->meshAttributes.method != MESH_TRANSFINITE ||
       CTX::instance()->mesh.flexibleTransfinite) &&
@@ -367,9 +395,11 @@ static void filterPoints(GEdge *ge, int nMinimumPoints)
     }
   }
 
+  if(!ge->getBeginVertex() || !ge->getEndVertex()) return;
+
   MVertex *v0 = ge->getBeginVertex()->mesh_vertices[0];
   std::vector<std::pair<double, MVertex *> > lengths;
-  for(unsigned int i = 0; i < ge->mesh_vertices.size(); i++) {
+  for(std::size_t i = 0; i < ge->mesh_vertices.size(); i++) {
     MEdgeVertex *v = dynamic_cast<MEdgeVertex *>(ge->mesh_vertices[i]);
     if(!v) {
       Msg::Error("in 1D mesh filterPoints");
@@ -431,6 +461,8 @@ static void filterPoints(GEdge *ge, int nMinimumPoints)
 static void createPoints(GVertex *gv, GEdge *ge, BoundaryLayerField *blf,
                          std::vector<MVertex *> &v, const SVector3 &dir)
 {
+  if(!ge->getBeginVertex() || !ge->getEndVertex()) return;
+
   const double hwall = blf->hwall(gv->tag());
   double L = hwall;
   double LEdge = distance(ge->getBeginVertex()->mesh_vertices[0],
@@ -462,6 +494,8 @@ static void addBoundaryLayerPoints(GEdge *ge, double &t_begin, double &t_end,
 
   if(n == 0) return;
 
+  if(!ge->getBeginVertex() || !ge->getEndVertex()) return;
+
   // Check if edge is a BL edge
   for(int i = 0; i < n; ++i) {
     Field *bl_field = fields->get(fields->getBoundaryLayerField(i));
@@ -530,10 +564,16 @@ void meshGEdge::operator()(GEdge *ge)
 
   ge->model()->setCurrentMeshEntity(ge);
 
-  if(ge->degenerate(1)) return;
-  if(ge->geomType() == GEntity::BoundaryLayerCurve) return;
-  if(ge->meshAttributes.method == MESH_NONE) return;
-  if(CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility()) return;
+  if (ge->degenerate(1)) {
+    ge->meshStatistics.status = GEdge::DONE;
+    return;
+  }
+
+  if ((ge->geomType() == GEntity::BoundaryLayerCurve) ||
+     (ge->meshAttributes.method == MESH_NONE) ||
+     (CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility())) {
+    return;
+  }
 
   deMeshGEdge dem;
   dem(ge);
@@ -543,34 +583,14 @@ void meshGEdge::operator()(GEdge *ge)
   if(ge->getMeshMaster() != ge) {
     GEdge *gef = dynamic_cast<GEdge *>(ge->getMeshMaster());
     if(gef->meshStatistics.status == GEdge::PENDING) return;
-    Msg::Info("Meshing curve %d (%s) as a copy of %d", ge->tag(),
+    Msg::Info("Meshing curve %d (%s) as a copy of curve %d", ge->tag(),
               ge->getTypeString().c_str(), ge->getMeshMaster()->tag());
     copyMesh(gef, ge, ge->masterOrientation);
     ge->meshStatistics.status = GEdge::DONE;
     return;
   }
 
-  if(ge->model()->getNumEdges() > 100000) {
-    if(ge->tag() % 100000 == 1) {
-      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(),
-                ge->model()->getNumEdges(), ge->getTypeString().c_str());
-    }
-  }
-  else if(ge->model()->getNumEdges() > 10000) {
-    if(ge->tag() % 10000 == 1) {
-      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(),
-                ge->model()->getNumEdges(), ge->getTypeString().c_str());
-    }
-  }
-  else if(ge->model()->getNumEdges() > 1000) {
-    if(ge->tag() % 1000 == 1) {
-      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(),
-                ge->model()->getNumEdges(), ge->getTypeString().c_str());
-    }
-  }
-  else {
-    Msg::Info("Meshing curve %d (%s)", ge->tag(), ge->getTypeString().c_str());
-  }
+  Msg::Info("Meshing curve %d (%s)", ge->tag(), ge->getTypeString().c_str());
 
   // compute bounds
   Range<double> bounds = ge->parBounds(0);
@@ -592,7 +612,8 @@ void meshGEdge::operator()(GEdge *ge)
     length = 0.0; // special case to avoid infinite loop in integration
   else
     length = Integration(ge, t_begin, t_end, F_One(), Points,
-                         1.e-12 * CTX::instance()->lc);
+                         CTX::instance()->mesh.lcIntegrationPrecision *
+                         CTX::instance()->lc);
   ge->setLength(length);
   Points.clear();
 
@@ -640,15 +661,11 @@ void meshGEdge::operator()(GEdge *ge)
     }
     if(CTX::instance()->mesh.algo2d != ALGO_2D_BAMG)
       a = smoothPrimitive(ge, std::sqrt(CTX::instance()->mesh.smoothRatio),
-                          Points);
+			  Points);
     filterMinimumN = ge->minimumMeshSegments() + 1;
     N = std::max(filterMinimumN, (int)(a + 1.99));
   }
 
-  if (ge->tag() == 236){
-    printf("%g %d\n",length,N);
-  }
-  
   // force odd number of points if blossom is used for recombination
   if((ge->meshAttributes.method != MESH_TRANSFINITE ||
       CTX::instance()->mesh.flexibleTransfinite) &&
@@ -682,8 +699,8 @@ void meshGEdge::operator()(GEdge *ge)
   std::vector<MVertex *> &mesh_vertices = ge->mesh_vertices;
 
   GPoint beg_p, end_p;
-  if(!ge->getBeginVertex() && !ge->getEndVertex()) {
-    Msg::Warning("Skipping curve with no begin nor end vertex");
+  if(!ge->getBeginVertex() || !ge->getEndVertex()) {
+    Msg::Warning("Skipping curve with no begin or end point");
     return;
   }
   else if(ge->getBeginVertex() == ge->getEndVertex() &&
@@ -735,7 +752,7 @@ void meshGEdge::operator()(GEdge *ge)
     std::vector<MVertex *> vv;
     vv.insert(vv.end(), _addBegin.begin(), _addBegin.end());
     vv.insert(vv.end(), mesh_vertices.begin(), mesh_vertices.end());
-    for(unsigned int i = 0; i < _addEnd.size(); i++)
+    for(std::size_t i = 0; i < _addEnd.size(); i++)
       vv.push_back(_addEnd[_addEnd.size() - 1 - i]);
     //    vv.insert(vv.end(), _addEnd.rend(), _addEnd.rbegin());
     mesh_vertices = vv;
@@ -771,6 +788,6 @@ void orientMeshGEdge::operator()(GEdge *ge)
 {
   // apply user-specified mesh orientation constraints
   if(ge->meshAttributes.reverseMesh)
-    for(unsigned int k = 0; k < ge->getNumMeshElements(); k++)
+    for(std::size_t k = 0; k < ge->getNumMeshElements(); k++)
       ge->getMeshElement(k)->reverse();
 }
