@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -274,6 +274,11 @@ public:
                         bool saveInterpolationMatrices = true,
                         bool forceNodeData = false,
                         bool forceElementData = false);
+  bool readCGNS(const std::pair<std::string, std::string> &solFieldName,
+                const std::string &fileName, int index, int fileIndex,
+                int baseIndex,
+                const std::vector<std::vector<MVertex *> > &vertPerZone,
+                const std::vector<std::vector<MElement *> > &eltPerZone);
   bool readMED(const std::string &fileName, int fileIndex);
   bool writeMED(const std::string &fileName);
   bool readPCH(const std::string &fileName, int fileIndex);
