@@ -10,7 +10,7 @@ gmsh.option.setNumber("General.Terminal", 1)
 gmsh.open(sys.argv[1])
 
 # Second arg force optimization of discrete volumes
-gmsh.model.mesh.optimize('Optimize', True)
-# gmsh.model.mesh.optimize('OptimizeNetgen', True)
+gmsh.model.mesh.optimize('', True)
+# gmsh.model.mesh.optimize('Netgen', True)
 
 gmsh.write('opt.msh')
